
/*
 * Quick and dirty MIDI file dumper
 */

import java.io.File;

import javax.sound.midi.MidiEvent;
import javax.sound.midi.MidiMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiFileFormat;
import javax.sound.midi.Sequence;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.Track;

public class QuickDump {

  public static void main(String[] args) throws Exception {
    String midiFileName = "" ;
    String outputFileName = "" ;
    boolean verbose = false ;
    int channelNumber = 10 ;

    if (args.length <= 0) {
      System.err.println("Missing command line arguments") ;
    }

    for (int i = 0 ; i < args.length ; i++) {
      switch( args[i].charAt(0)) {
      case '0' :
      case '1' :
      case '2' :
      case '3' :
      case '4' :
      case '5' :
      case '6' :
      case '7' :
      case '8' :
      case '9' :
	// Channel number
	break ;
      case '-' :
	  switch( args[i].charAt(1)) {
	  case 'v' :
	      // Verbose flag
	      verbose = true ;
	      break ;
	  default :
	      System.err.println("Unknown option") ;
	      break ;
	  }
	  break ;
      default :
	// Assume it's the MIDI file name
        midiFileName = args[i] ;
	break ;
      }
    }

 
    outputFileName = "_" +  midiFileName ;

    if (verbose) {
      System.out.println("MIDI file:   " + midiFileName) ;
      System.out.println("Output file: " + outputFileName) ;
      for (String s: args) {
        System.out.println(s);
      }
    }

    if (midiFileName != "") {
      File midiFile = new File(midiFileName) ;
      MidiFileFormat midiFileFormat = MidiSystem.getMidiFileFormat(midiFile) ;
      int fileType = midiFileFormat.getType() ;
      float divisionType = midiFileFormat.getDivisionType() ;
      int resolution = midiFileFormat.getResolution() ;
      int byteLength = midiFileFormat.getByteLength() ;
      long microsecondLength = midiFileFormat.getMicrosecondLength() ;

      System.out.println("File type:     " + fileType) ;
      System.out.println("File length:   " + byteLength) ;
      System.out.println("Time length:   " + (microsecondLength/100000)) ;
      System.out.println("Resolution:    " + resolution) ;
      System.out.println("Division type: " + divisionType) ;

      Sequence sequence = MidiSystem.getSequence(midiFile);
      QuickDumper nm = new QuickDumper(sequence) ;
      nm.dumpSequence() ;
      // MidiSystem.write(sequence, fileType, new File(outputFileName)) ;
    }
  }
}
