
/*
 * Quick and dirty MIDI sequence dumper
 */

import java.io.File;

import javax.sound.midi.MidiEvent;
import javax.sound.midi.MidiMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.Sequence;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.Track;

public class QuickDumper {
  private Sequence sequence ;

  private static final int NOTE_ON = 0x90 ;
  private static final int NOTE_OFF = 0x80 ;
  private static final int CC = 0xB0 ;
  private static final int PC = 0xC0 ;
  private static final int CAT = 0xD0 ;
  private static final int PAT = 0xD0 ;
  private static final int PB = 0xD0 ;
  private static final int MIDI_CLOCK = 0xF8 ;
  private static final int MIDI_START = 0xFA ;
  private static final int MIDI_CONTINUE = 0xFB ;
  private static final int MIDI_STOP = 0xFC ;
  private static final int MIDI_ACTIVE_SENSE = 0xFE ;
  private static final int MIDI_SYSTEM_RESET = 0xFE ;

  private static final String[] NOTE_NAMES = 
  {
    "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"
  } ;

  public QuickDumper(Sequence s) {
    sequence = s ;
  }

  private void handleShortMessage(ShortMessage sm) {
    System.out.print("Ch: " + sm.getChannel() + " ");
    if (sm.getCommand() == NOTE_ON) {
      int key = sm.getData1();
      int octave = (key / 12)-1;
      int note = key % 12;
      String noteName = NOTE_NAMES[note];
      int velocity = sm.getData2();
      System.out.println("NOTE ON  " + noteName + octave + " Key:" + key + " Vel: " + velocity);
    } else if (sm.getCommand() == NOTE_OFF) {
      int key = sm.getData1();
      int octave = (key / 12)-1;
      int note = key % 12;
      String noteName = NOTE_NAMES[note];
      int velocity = sm.getData2();
      System.out.println("NOTE OFF " + noteName + octave + " Key=" + key + " Vel: " + velocity);
    } else if (sm.getCommand() == CC) {
      int ctlr = sm.getData1();
      int value = sm.getData2();
      System.out.println("CC CTLR:" + ctlr + " Value: " + value) ;
    } else if (sm.getCommand() == PB) {
      int lsb = sm.getData1();
      int msb = sm.getData2();
      System.out.println("PB MSB:" + msb + " LSB: " + lsb) ;
    } else if (sm.getCommand() == CAT) {
      int value = sm.getData1();
      System.out.println("CAT Value: " + value) ;
    } else if (sm.getCommand() == PAT) {
      int key = sm.getData1();
      int value = sm.getData2();
      System.out.println("PAT Key:" + key + " value: " + value) ;
    } else {
      System.out.println("Command:" + Integer.toHexString(sm.getCommand()));
    }
  }

  private void scanOneTrack(Track track) {
    for (int i=0; i < track.size(); i++) { 
      MidiEvent event = track.get(i) ;
      System.out.print("@" + event.getTick() + " ");
      MidiMessage message = event.getMessage() ;
      if (message instanceof ShortMessage) {
        ShortMessage sm = (ShortMessage) message ;
	handleShortMessage(sm) ;
      } else {
        // Long MIDI messages (SysEx, Meta)
        System.out.println("Long message: " + message.getClass()) ;
      }
    }
    System.out.println();
  }

  private void scanTracks() {
    int trackNumber = 0;
    for (Track track :  sequence.getTracks()) {
      trackNumber++;
      System.out.println("Track " + trackNumber + ": size = " + track.size());
      System.out.println();
      scanOneTrack(track) ;
     }
  }

  public void dumpSequence() {
    scanTracks() ;
  }
}
